// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "riegeli/digests/crc32_digester.h"

#include <stdint.h>

#include <limits>

#include "absl/strings/string_view.h"
#include "riegeli/base/arithmetic.h"
#include "zlib.h"

namespace riegeli {

void Crc32Digester::Write(absl::string_view src) {
  while (src.size() > std::numeric_limits<uInt>::max()) {
    crc_ = IntCast<uint32_t>(crc32(IntCast<uLong>(crc_),
                                   reinterpret_cast<const Bytef*>(src.data()),
                                   std::numeric_limits<uInt>::max()));
    src.remove_prefix(std::numeric_limits<uInt>::max());
  }
  crc_ = IntCast<uint32_t>(crc32(IntCast<uLong>(crc_),
                                 reinterpret_cast<const Bytef*>(src.data()),
                                 IntCast<uInt>(src.size())));
}

}  // namespace riegeli
